@@ -4,7 +4,7 @@ use 5.010;
 use strict;
 use warnings;
 use base 'Net::Server::PreFork';
-our $VERSION = '2.20181024'; # VERSION
+our $VERSION = '2.20181114'; # VERSION
 
 
 use English qw{ -no_match_vars };
@@ -36,6 +36,32 @@ sub _warn {
 }
 
 
+sub preload_modules {
+    my ( $self, $from, $matching ) = @_;
+    my $installed = ExtUtils::Installed->new( 'skip_cwd' => 1 );
+
+    my $path_matching = $matching;
+    $path_matching =~ s/::/\//g;
+    foreach my $module ( grep { /$from/ } $installed->modules() ) {
+        FILE:
+        foreach my $file ( grep { /$path_matching\/\w+\.pm$/ } $installed->files( $module ) ) {
+            next FILE if ! -e $file;
+            my ( $module ) = reverse split '/', $file;
+            $module =~ s/\.pm$//;
+            $module = join( '::', $matching, $module );
+            if ( ! is_loaded( $module ) ) {
+                $self->logdebug( "Preloading Module $module" );
+                load $module;
+            }
+            else {
+                $self->logdebug( "Preloading Module $module already loaded" );
+            }
+        }
+    }
+    return;
+}
+
+
 sub get_installed_handlers {
     my @installed_handlers;
     my $installed = ExtUtils::Installed->new( 'skip_cwd' => 1 );
@@ -64,6 +90,7 @@ sub pre_loop_hook {
 
     $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':master';
 
+    $self->preload_modules( 'Net::DNS', 'Net::DNS::RR' );
     $self->{'metric'} = Mail::Milter::Authentication::Metric->new();
 
     # Load handlers
@@ -1035,7 +1062,7 @@ Mail::Milter::Authentication - A Perl Mail Authentication Milter
 
 =head1 VERSION
 
-version 2.20181024
+version 2.20181114
 
 =head1 SYNOPSIS
 
@@ -1057,6 +1084,12 @@ A Perl Implementation of email authentication standards rolled up into a single
 
 =head1 METHODS
 
+=head2 preload_modules( $from, $matching )
+
+Preload (pre-fork) lazy loading modules.
+
+Takes a Package Name and a Base module, and loads all modules which match.
+
 =head2 I<idle_loop_hook()>
 
 Hook which runs in the master periodically.
